#pragma once

#include "VideoModel.hpp"
#include "core.hpp"
#include <gtkmm/window.h>
#include <gtkmm/applicationwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/stack.h>
#include <gtkmm/stackswitcher.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/listview.h>
#include <gtkmm/singleselection.h>
#include <giomm/liststore.h>
#include <gtkmm/listitem.h>
#include <gtkmm/paned.h>
#include <gtkmm/menubutton.h>
#include <gtkmm/builder.h>

namespace
{
    struct PlayButtonParams
    {
        // Какой id видео
        std::string yt_id;
        // Кнопка
        Gtk::Button* caller;
    };

    struct RateButtonParams
    {
        // Какая оценка
        core:: VideoRating rating;
    };
}

namespace components {

    // Главное окно приложения
    class MainWindow : public Gtk::ApplicationWindow
    {
    public:
        static MainWindow* create(core::Core* appcore);
        MainWindow(
            BaseObjectType* cobject,
            const Glib::RefPtr<Gtk::Builder>& refBuilder,
            core::Core* appcore);

    protected:
        core::Core* m_appcore;
        Glib::RefPtr<Gtk::Builder> m_builder;
        Gtk::Box m_layout;
        Gtk::Box m_head;
        Gtk::Stack* m_body;
        Gtk::StackSwitcher* m_stackswitcher;

        /// ПОИСК ///
        // Тело поиска
        Gtk::Paned* m_search_body;
        // Поле ввода запроса
        Gtk::Entry* m_search_entry;
        // Поле ввода запроса на стартовой странице
        Gtk::Entry* m_start_search_entry;
        // Список моделей видео
        Glib::RefPtr<Gio::ListStore<components::VideoModel>> m_video_storage;
        // Модель выбора
        Glib::RefPtr<Gtk::SingleSelection> m_selection_model;

        /// ДЕТАЛИ ВИДЕО ///
        Gtk::Box m_rate_container;
        Gtk::Button* m_rate_dislike_btn;
        Gtk::Button* m_rate_meh_btn;
        Gtk::Button* m_rate_like_btn;

        // --События--
        // Клик кнопки поиска
        void onSearchButtonClicked();
        // Клик кнопки поиска на стартовой странице
        void onStartSearchButtonClicked();
        // При создании элемента в m_video_list
        void onVideoSetup(const Glib::RefPtr<Gtk::ListItem>& list_item);
        // При записи данных в m_video_list
        void onVideoBind(const Glib::RefPtr<Gtk::ListItem>& list_item);
        // При выборе видео из списка
        void onVideoChanged();
        // При клике на кнопку "Воспроизвести"
        void onPlayButtonClicked(PlayButtonParams e);
        // При клике на кнопку оценки видео
        void onRateButtonClicked(RateButtonParams e);

        // --Общее--
        // Оценка выбранного видео
        int m_current_rating;

        // Устанавливает боковую панель просмотра видео в обычный режим
        void searchSidebarRegularMode();

        // Устанавливает боковую панель просмотра видео в обычный режим
        void searchSidebarError(Glib::ustring description);

        // Устанавливает боковую панель просмотра видео в режим просмотра
        // информации о видео
        // title - Название видео
        // channel_title - Название канала
        // description - Описание видео
        // published_at - Дата публикации
        // yt_id - id видео на YouTube
        // views_count - количество просмотров
        // thumbnail_url - ссылка на большое превью
        void searchSidebarVideoDetailsMode(
            Glib::ustring title,
            Glib::ustring channel_title,
            Glib::ustring description,
            std::string published_at,
            std::string yt_id,
            int views_count,
            std::string thumbnail_url);

        // Проигрывает конкретное видео
        void playSingleVideo(std::string yt_id);
        // Открывает страницу видео в YouTube
        void openVideoInWebBrowser(std::string yt_id);
        // Обновляет текущие переменные при просмотре информации о видео
        void updateVideoDetails(std::string yt_id, core::VideoRating rating);
        // Выполняет основную часть поиска видео
        void searchVideoByQuery(Glib::ustring query);
    };
}