#pragma once

#include "core.hpp"
#include <gtkmm/window.h>
#include <glibmm.h>
#include <gtkmm/builder.h>
#include <gtkmm/dropdown.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/checkbutton.h>

namespace {
    // Настройки которые можно настроить булевыми переменными
    enum class BoolSettingType {
        MINIPLAYER,
        FULLSCREEN,
        ONTOP
    };

    // Параметры клика кнопок да/нет
    struct BoolButtonParams {
        BoolSettingType setting_type;
        Gtk::CheckButton* caller;
    };
}

namespace components {
    class PrefWindow : public Gtk::Window
    {
    public:
        static PrefWindow* create(Gtk::Window& parent, core::Core* appcore);
        PrefWindow(
            BaseObjectType* cobject,
            const Glib::RefPtr<Gtk::Builder>& refBuilder,
            core::Core* appcore);

    protected:
        core::Core* m_appcore;
        Glib::RefPtr<Gtk::Builder> m_builder;
        Gtk::DropDown* m_quality_dropdown;
        Gtk::Entry* m_apikey_entry;
        Gtk::Entry* m_mpvpath_entry;
        Gtk::Button* m_cancel_btn;
        Gtk::Button* m_save_btn;
        Gtk::CheckButton* m_miniplayer_btn;
        Gtk::CheckButton* m_fullscreen_btn;
        Gtk::CheckButton* m_ontop_btn;

        // -- Обработчики событий --
        void onCancelClicked();
        void onSaveClicked();
        void onBoolButtonClicked(BoolButtonParams params);

        // Сохраняет настройки
        void saveConfig();

        // Выполняет подготовку кнопки переключателя
        void prepareBoolButton(
            Gtk::CheckButton* widget,
            bool default_value,
            BoolSettingType setting_type);
    };
}
