#include "app.hpp"
#include <iostream>
#include <giomm/menu.h>
#include <gtkmm/builder.h>
#include <assert.h>
#include <sstream>

namespace components {

    Glib::RefPtr<App> App::create(core::Core* appcore)
    {
        auto app = Glib::make_refptr_for_instance<App>(new App(appcore));
        return app;
    }

    App::App(core::Core* appcore)
        : Gtk::Application("com.github.aquadim.YTMPV")
        , m_appcore(appcore)
    {}

    void App::on_startup()
    {
        Gtk::Application::on_startup();

        add_action("preferences", sigc::mem_fun(*this, &App::onActionPrefs));
        set_accel_for_action("app.preferences", "<Ctrl>P");

        add_action("quit", sigc::mem_fun(*this, &App::onActionQuit));
        set_accel_for_action("app.quit", "<Ctrl>Q");

        add_action("copyytid", sigc::mem_fun(*this, &App::onActionCopyYTID));
        set_accel_for_action("app.copyytid", "<Ctrl>I");

        add_action("copyytlink", sigc::mem_fun(*this, &App::onActionCopyYTLink));
        set_accel_for_action("app.copyytlink", "<Ctrl>L");

        // Установка менюбара
        auto menu_builder = Gtk::Builder::create_from_resource("/src/YTMPV/res/MainMenu.ui");
        auto menu = menu_builder->get_object<Gio::Menu>("menu");
        assert(menu != nullptr);
        set_menubar(menu);
    }

    void App::on_activate()
    {
        m_main_window = createMainWindow();
        m_main_window->present();
    }

    void App::onActionQuit()
    {
        auto windows = get_windows();
        for (auto window : windows)
        {
            window->set_visible(false);
        }
        quit();
    }
    
    void App::onActionPrefs()
    {
        PrefWindow* win = createPrefWindow();
        win->present();
    }

    void App::onActionCopyYTID()
    {
        m_main_window->get_clipboard()->set_text(
            m_appcore->getSelectedYTID());
    }

    void App::onActionCopyYTLink()
    {
        m_main_window->get_clipboard()->set_text(
            m_appcore->getSelectedYTLink());
    }
    
    MainWindow* App::createMainWindow()
    {
        // Создание окна и привязка обработчика к его сигналу скрытия
        auto appwindow = MainWindow::create(m_appcore);
        add_window(*appwindow);
        appwindow->signal_hide().connect(
            [appwindow](){ delete appwindow; });
        return appwindow;
    }

    PrefWindow* App::createPrefWindow()
    {
        auto prefwindow = PrefWindow::create(
            *get_active_window(),
            m_appcore
        );
        add_window(*prefwindow);
        prefwindow->signal_hide().connect(
            [prefwindow](){ delete prefwindow; });
        return prefwindow;
    }
}