#pragma once

#include <YoutubeApi/youtubeapi.hpp>
#include <Database/entities.hpp>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <chrono>

// Реализация приложения без графики
// Содержит функции и переменные, которые необходимы во всех компонентах
// Класс отделён от класса App для улучшенной поддержки тестирования
// TODO: тестирование класса

namespace core
{

    enum class VideoRating
    {
        NONE,
        LIKE,
        DISLIKE,
    };

    class Core
    {
        public:
            Core(
                std::string apikey,
                std::string config_path,
                int quality,
                bool launch_fullscreen,
                bool launch_ontop,
                bool launch_miniplayer,
                std::string mpvpath,
                std::string thumbnails_dir
            );

            //{ Установщики
            // Устанавливает качество видео
            void setQuality(int quality);

            // Устанавливает путь к MPV
            void setMpvPath(std::string path);

            // Устанавливает выбранное видео
            void setSelectedYTID(std::string ytid);

            // Устанавливает оценку выбранного видео
            void setCurrentRating(VideoRating rating);

            // Устанавливает режим мини-проигрывателя
            void setMiniplayer(bool value);

            // Устанавливает режим полного экрана
            void setFullscreen(bool value);

            // Устанавливает режим "Поверх других окон"
            void setOnTop(bool value);

            // Устанавливает api key
            void setApiKey(std::string value);
            //}

            //{ Возвращатели
            // Возвращает ytid выбранного видео
            std::string getSelectedYTID();

            // Возвращает ссылку на выбранное видео
            std::string getSelectedYTLink();

            // Возвращает качество видео
            int getQuality();

            // Возвращает путь к MPV
            std::string getMpvPath();

            // Возвращает ключ API
            std::string getApiKey();

            // Возвращает параметр запуска в мини-плеере
            bool getIsMiniplayer();

            // Возвращает режим полноэкранного запуска
            bool getFullscreen();

            // Возвращает режим запуска поверх других окон
            bool getOnTop();
            //}

            // Сохраняет настройки в файл по пути m_config_path
            void saveConfig();

            // Возвращает объект видео по его ytid
            db::video getVideoByYTID(std::string ytid);

            // Возвращает строку формата yt-dl в соответствии с m_quality
            std::string getYTDLFormat();

            // Оценивает видео
            // rating: 0 - нет, 1 - лайк, 2 - дизлайк
            // Если оценка текущего видео такова же как и устанавливаемая,
            // запрос не произойдёт в целях экономии квоты API
            void rateVideo(std::string ytid, VideoRating rating);

            // Возвращает параметры запуска MPV
            std::vector<std::string> getMPVLaunchParams(std::string ytid);

            // Возвращает ссылку на видео по его ytid
            std::string getYTLink(std::string ytid);

            // Возвращает список из ytid по запросу
            ytapi::SearchResults searchVideos(std::string query);

            // Возвращает путь к маленькому превью видео по его ytid
            std::string getSmallThumbnailByYTID(std::string ytid);

            // Возвращает путь к большому превью видео по его ytid
            std::string getLargeThumbnailByYTID(std::string ytid);

            // Возвращает сгруппированое число прописью
            std::string getNumberGrouped(int n);

            // Возвращает unix-время от времени публикации видео
            std::string getDateText(std::string date);

        protected:
            // ключ API
            std::string m_apikey;

            // ytid "выбранного" видео
            // Необходимо для:
            // - Запуска MPV
            // - Копирования ссылки видео
            // - Открытия веб-страницы видео
            std::string m_selected_ytid;

            VideoRating m_current_rating;

            // Путь к файлу настроек
            std::string m_config_path;

            // С каким качеством запускать видео
            // Значение в переменной подставится в параметр запуска MPV
            // --ytdl-format
            int m_quality;

            // Запускать ли MPV в полноэкранном режиме по умолчанию?
            bool m_launch_fullscreen;

            // Запускать ли MPV над всеми окнами?
            bool m_launch_ontop;

            // Запускать ли MPV над всеми окнами, в углу экрана?
            // Имеет приоритет над m_launch_fullscreen и m_launch_ontop
            bool m_launch_miniplayer;

            // Настройка пути к MPV
            std::string m_mpvpath;

            // Путь к директории кэшированных превью видео
            std::string m_thumbnails_dir;
    };
}
