#include "app.hpp"
#include "core.hpp"
#include <glibmm.h>
#include <curl/curl.h>
#include <Database/database.hpp>
#include <YoutubeApi/youtubeapi.hpp>
#include <iostream>
#include <fstream>
#include <nlohmann/json.hpp>

// TODO:
/*
 * 1. Окно настройки
 * 2. oAuth
 * 3. Оценка видео
 * 4. Проверка подключения (curl возвращает пустые строки json)
*/

using json = nlohmann::json;

int main(int argc, char** argv) {
    // -- Локаль --
    std::locale::global(std::locale(""));

    // -- Конструирование путей файлов --
    std::string config_path = Glib::build_filename(
        Glib::get_user_config_dir(),
        "ytmpv",
        "config.json");

    /*std::string secrets_path = Glib::build_filename(
        Glib::get_user_config_dir(),
        "ytmpv",
        "secrets.json");*/

    std::string db_path = Glib::build_filename(
        Glib::get_user_cache_dir(),
        "ytmpv",
        "db.sqlite3");

    std::string thumbnails_dir = Glib::build_filename(
        Glib::get_user_cache_dir(),
        "ytmpv",
        "thumbnails");
    
    std::ifstream config_file(config_path);
    auto j_config = json::parse(config_file);
    config_file.close();

    /*std::ifstream secrets_file(secrets_path);
    auto j_secrets = json::parse(secrets_file);
    secrets_file.close();*/
    
    // -- Общая инициализация --
    curl_global_init(CURL_GLOBAL_ALL);
    db::databaseInit(db_path);
    db::createTables();

    // -- Запуск приложения --
    core::Core appcore = core::Core(
        j_config["apikey"],
        config_path,
        j_config["quality"],
        j_config["fullscreen"],
        j_config["ontop"],
        j_config["miniplayer"],
        j_config["mpvpath"],
        thumbnails_dir
    );
    auto app = components::App::create(&appcore);
    return app->run(argc, argv);
}
